
----------------------------------------
-- Variables
----------------------------------------

local addonName, BGS_FrameInfoTable = ...
local versionNr = GetAddOnMetadata(addonName, "Version")
BGS_FrameInfoTable.ExtraInfoFunctions = {functions ={}, lists = {}}

local AceGUI = LibStub("AceGUI-3.0")

local _
local ExtraDataList = {}
local DefaultFrames = {}
local OptionsFrame = {}
local debugMode = true -- disable debug options (personal use when fixing the addon)


local DEFAULT_TRACKOFFSET_WITH = 27
local DEFAULT_TRACKOFFSET_WITHOUT = 10
local DEFAULT_LOCKVERTEX_OFF = 0.5
local DEFAULT_LOCKVERTEX_ON = 0.8
local OPTIONS_TINYBTN_SIZE = 20

local DEFAULT_BG = "Interface\\DialogFrame\\UI-DialogBox-Background"
local DEFAULT_EDGEFILE = "Interface\\DialogFrame\\UI-DialogBox-Border"

local mainBGBack = "Interface\\DialogFrame\\UI-DialogBox-Background"
local mainEdgefile = nill


local iconOffsetX = 6
local iconOffsetY = -1

local BGTable = {["none"] = "None"
				,["Interface\\COMMON\\ShadowOverlay-Top"] = "Fade Top"
				,["Interface\\COMMON\\ShadowOverlay-Right"] = "Fade Right"
				,["Interface\\COMMON\\ShadowOverlay-Bottom"] = "Fade Bottom"
				,["Interface\\COMMON\\ShadowOverlay-Left"] = "Fade Left"
				,["Interface\\DialogFrame\\UI-DialogBox-Background"] = "Full Light"
				,["Interface\\DialogFrame\\UI-DialogBox-Background-Dark"] = "Full Dark"
				,["Interface\\DialogFrame\\UI-DialogBox-Gold-Background"] = "Full Gold"
				,["Interface\\Glues\\CHARACTERCREATE\\UI-CHARACTERCREATE-BACKGROUND"] = "Full Border"
				,["Interface\\BankFrame\\Bank-Background"] = "Stone Light"
				,["Interface\\RAIDFRAME\\UI-RaidFrame-GroupBg"] = "Stone Dark"
				,["Interface\\GuildBankFrame\\GuildVaultBG"] = "Stone Red"
				,["Interface\\Glues\\Models\\UI_NIGHTELF\\AA_NE_SKY"] = "Sky Night"
				,["Interface\\Glues\\Models\\UI_Orc\\MM_ORC_SKY_01"] = "Sky Dawn"
				,["Interface\\Glues\\Models\\UI_Orc\\MM_ORC_S_01"] = "Test"
				}

local BGOrderTable = {"none"
						,"Interface\\COMMON\\ShadowOverlay-Top"
						,"Interface\\COMMON\\ShadowOverlay-Right"
						,"Interface\\COMMON\\ShadowOverlay-Bottom"
						,"Interface\\COMMON\\ShadowOverlay-Left"
						,"Interface\\DialogFrame\\UI-DialogBox-Background"
						,"Interface\\DialogFrame\\UI-DialogBox-Background-Dark"
						,"Interface\\DialogFrame\\UI-DialogBox-Gold-Background"
						,"Interface\\Glues\\CHARACTERCREATE\\UI-CHARACTERCREATE-BACKGROUND"
						,"Interface\\BankFrame\\Bank-Background"
						,"Interface\\RAIDFRAME\\UI-RaidFrame-GroupBg"
						,"Interface\\GuildBankFrame\\GuildVaultBG"
						,"Interface\\Glues\\Models\\UI_NIGHTELF\\AA_NE_SKY"
						,"Interface\\Glues\\Models\\UI_Orc\\MM_ORC_SKY_01"
						}
				
local _holidayInfoList = {{full = "General Battlegrounds", icon = "Interface\\ICONS\\Achievement_BG_KillFlagCarriers_grabFlag_CapIt"}
						,{full = "Warsong Gulch", icon = "Interface\\ICONS\\INV_Misc_Rune_07"}
						,{full = "Arathi Basin", icon = "Interface\\ICONS\\INV_Jewelry_Amulet_07"}
						,{full = "Alterac Valley", icon = "Interface\\ICONS\\INV_Jewelry_Necklace_21"}
						,{full = "Eye of the Storm", icon = "Interface\\ICONS\\Spell_Nature_EyeOfTheStorm"}
						,{full = "Strand of the Ancients", icon = "Interface\\ICONS\\Achievement_BG_winSOA"}
						,{full = "Isle of Conquest", icon = "Interface\\ICONS\\Achievement_BG_winWSG"}
						,{full = "The Battle for Gilneas", icon = "Interface\\ICONS\\Achievement_Battleground_BattleForGilneas"}
						,{full = "Twin Peaks", icon = "Interface\\ICONS\\spell_nature_earthshock"}
						,{full = "Silvershard Mines", icon = "Interface\\ICONS\\INV_Crate_07"}
						,{full = "Temple of Kotmogu", icon = "Interface\\ICONS\\Achievement_Battleground_TempleOfKotmogu_02"}
						,{full = "Deepwind Gorge", icon = "Interface\\ICONS\\achievement_zone_valleyoffourwinds"}
						}
local FontCheckBox = "GameFontWhite"
local FontTracker = "GameFontNormal"

local BGOptions = { }
local ActiveBG = DEFAULT_BG

local TrackingFrames = {}


BGS_FrameEventFunctions = {}

local ddwn_Background
local _Winrateframe

local L_BGS_CBHideIcons = nil
local L_BGS_CBHideTitle = nil


----------------------------------------
-- Addon Wide functions
----------------------------------------

function BGS_FrameInfoTable:GetAccHKills()

-- Temp way of tracking account wide Hk. capped at 250k
local _, _, _, HKills, _, _, _, _, _, _ = GetAchievementCriteriaInfo(5363, 1)

--local HKills, _ = GetPVPLifetimeStats()
   return HKills
end

function BGS_FrameInfoTable:GetHKills()

   local HKills, _ = GetPVPLifetimeStats()
   return HKills
end

function BGS_FrameInfoTable:GetHonor()
   local _, CurrInfo, _, _, _, _ = GetCurrencyInfo(392)
   return CurrInfo
end

function BGS_FrameInfoTable:GetConquest()
   local _, CurrInfo, _, _, _, _ = GetCurrencyInfo(390)
   return CurrInfo
end

function BGS_FrameInfoTable:GetTB()
   local _, CurrInfo, _, _, _, _ = GetCurrencyInfo(391)
   return CurrInfo
end

function BGS_FrameInfoTable:GetBGHoliday()
	local _, _, bgName, _, bgWon, _, _, _, _, _ = GetHolidayBGInfo();
	if bgName ~= nil then
		return bgName
	end
	return "Unknown BG"
end

function BGS_FrameInfoTable:GetBGHolidayWon()
	local _, bgName, _, bgWon, _, _, _, _, _ = GetHolidayBGInfo();
	return bgWon
end

function BGS_FrameInfoTable:GetFrameByName(name)
	for k, v in ipairs(DefaultFrames) do
		if string.lower(v:GetName()) == string.lower("BGS_"..name) then
			return v
		end
	end
	
	return nil
end


local function TempOETSWinLossFix(list)

	table.remove(list, 4)-- remove EotS

	local wins = GetStatistic(list[1][3]) --total wins
	local total = GetStatistic(list[1][2]) --total losses
	local loss = 0
	
	if total == "--" or total == 0 or wins == "--" then
		wins = 0
		loss = 0
	else
		loss = total - wins
	end
	
	for k, v in ipairs(list) do
		if k ~= 1 then
			local bgw = GetStatistic(v[3])
			local bgt = GetStatistic(v[2])
			local bgl = 0
			
			if bgt == "--" or bgt == 0 or bgw == "--" then
				bgw = 0
				bgl = 0
			else
				bgl = bgt - bgw
			end
			
			wins = wins - bgw
			loss = loss - bgl
			
		end
	end
	
	return wins, loss
end

local function BGS_GetWinrateString(name)
	local text = ""
	local bg, total, win, loss
	local list = {{"General Battlegrounds", 839, 840},{"Warsong Gulch", 52, 105},{"Arathi Basin", 55, 51},{"Eye of the Storm", 54, 50},{"Alterac Valley", 53, 49}
		,{"Strand of the Ancients", 1549, 1550},{"Isle of Conquest", 4096, 4097},{"Twin Peaks", 5232, 5233},{"The Battle for Gilneas", 5236, 5237}
		,{"Temple of Kotmogu", 7825, 7826},{"Silvershard Mines", 7829, 7830},{"Deepwind Gorge", 8374, 8373}}
	--general
	if (name) then
		for k, v in ipairs(list) do
			if v[1] == name then
				bg = v[1]
				if v[1] ~= "Eye of the Storm" then 
					
				
					win = GetStatistic(v[3])
					total = GetStatistic(v[2])
					if total == "--" or total == 0 or win == "--" then
						winrate = 0
						loss = 0
					else
						winrate =  (win/total)*100
						loss = total - win
					end
				else
					win, loss = TempOETSWinLossFix(list)
					total = win + loss
					if total == 0 then
						winrate = 0
					else
						winrate =  (win/total)*100
					end
				end
				
				
			end
		end
		if (bg) then
			text = text .. string.format("|cFFCCFFCC%d|r : |cFFFFCCCC%d|r  (%.1f %%)", win, loss, winrate)
		else
			text = "Unknown Bg"
		end
	else
		for k, v in ipairs(list) do
		bg = v[1]
		
		win = GetStatistic(v[3])
		total = GetStatistic(v[2])
		if total == "--" or total == 0 or win == "--" then
			winrate = 0
			loss = 0
		else
			winrate =  (win/total)*100
			loss = total - win
		end
		--text = text .. string.format("|cFFFFD100%s:\nTotal:|r %-5d |cFFFFD100Wins:|r %-5d |cFFFFD100Winrate:|r %.1f %%\n\n",bg, total, win, winrate)
		text = text .. string.format("|cFFFFD100%s:|r\n |cFFCCFFCC%d|r : |cFFFFCCCC%d|r  (%.1f %%)",bg, win, loss, winrate)
		if bg == "Eye of the Storm" then
			text = text .. " |cFFFFD100*|r"
		end
		
		if k ~= table.getn(list) then
			text = text .. "\n\n"
		end
		end
	
		text = text .. "\n\n|cFFFFD100* bugged in blizz code|r"
		
	end
	return text
end

local function _UpdateWinrate()

	for k, bg in ipairs(_holidayInfoList) do
		bg.frame.text:SetText("|cFFFFD100"..bg.full..":|r\n " ..BGS_GetWinrateString(bg.full))
	end
	
	
end

local function GetOptionsFrameByInfo(info)
	for k, v in ipairs(OptionsFrame) do
		if string.lower(v.text:GetText()) == string.lower(info) then
			return v
		end
	end
	
	return -1
end

local function SetShowMainframe(show) 
	if show then
		IsShown = true
		BGStatFrame:Show()
		BGS_VisibleButton.tex:SetTexture("Interface\\LFGFRAME\\BattlenetWorking9")
	else
		IsShown = false
		BGStatFrame:Hide()
		BGS_VisibleButton.tex:SetTexture("Interface\\LFGFRAME\\BattlenetWorking4")
	end
end

local function UpdateMainFrameBG()
	if BGStatFrame:IsMouseEnabled() then
		mainEdgefile = DEFAULT_EDGEFILE
	else
		mainEdgefile = nill
	end
	BGStatFrame:SetBackdrop({bgFile = mainBGBack,
      edgeFile = mainEdgefile,
	  tileSize = 0, edgeSize = 16,
      insets = { left = 3, right = 3, top = 3, bottom = 3 }
	  })
end

local function ToggleLockbutton() 
	if BGStatFrame:IsMouseEnabled() then
		BGS_MoveButton.tex:SetVertexColor(DEFAULT_LOCKVERTEX_OFF, DEFAULT_LOCKVERTEX_OFF, DEFAULT_LOCKVERTEX_OFF )
		PlaySound("igMainMenuOptionCheckBoxOff");
		BGStatFrame:EnableMouse(false)
		BGS_LockButton.tex:SetVertexColor(DEFAULT_LOCKVERTEX_OFF, DEFAULT_LOCKVERTEX_OFF, DEFAULT_LOCKVERTEX_OFF )
			
	else	
		BGS_MoveButton.tex:SetVertexColor(DEFAULT_LOCKVERTEX_ON, DEFAULT_LOCKVERTEX_ON, DEFAULT_LOCKVERTEX_ON )
		PlaySound("igMainMenuOptionCheckBoxOn");
		BGStatFrame:EnableMouse(true)
		BGS_LockButton.tex:SetVertexColor(DEFAULT_LOCKVERTEX_ON, DEFAULT_LOCKVERTEX_ON, DEFAULT_LOCKVERTEX_ON )
			
	end
		UpdateMainFrameBG()
end

local function FactionIcon()
   local EnglishFaction, localizedFaction = UnitFactionGroup("player")
   if EnglishFaction == "Horde" then
      BGS_Icon:SetBackdrop({
            bgFile = [[Interface\PVPFrame\PVP-Currency-Horde]],
			--bgFile = "Interface\\Timer\\Horde-Logo",
            edgeFile = nil,
            tileSize = 32, edgeSize = 0,
      })
   else
      BGS_Icon:SetBackdrop({
            bgFile = [[Interface\PVPFrame\PVP-Currency-Alliance]],
			--bgFile = "Interface\\Timer\\Alliance-Logo",
            edgeFile = nil,
            tileSize = 32, edgeSize = 0,
      })
   end
end



local function _CreateBaseFrame()
-- Mainframe
------------------------------------------------------------------------------------------------------------------------
	local L_BGStatFrame = CreateFrame("frame", "BGStatFrame", UIParent)
		UpdateMainFrameBG()
	BGStatFrame:SetWidth(107)
	BGStatFrame:SetHeight(300)
	BGStatFrame:SetMovable(true)
	BGStatFrame:SetPoint("Center", 200, 0)
	BGStatFrame:RegisterForDrag("LeftButton")
	BGStatFrame:SetScript("OnDragStart", BGStatFrame.StartMoving )
	BGStatFrame:SetScript("OnDragStop", BGStatFrame.StopMovingOrSizing)
	BGStatFrame:SetClampedToScreen(true)
	BGStatFrame:Show()

-- Mainframe Title
------------------------------------------------------------------------------------------------------------------------
	local L_FrameTitle = CreateFrame("Frame", "BGS_FrameTitle", BGStatFrame)
	BGS_FrameTitle:SetPoint("top", BGStatFrame, "top", 0, 0)
	BGS_FrameTitle:SetWidth(BGStatFrame:GetWidth())
	BGS_FrameTitle:SetHeight(33)
	BGS_FrameTitle:Show()

-- Mainframe Faction Icon
------------------------------------------------------------------------------------------------------------------------
	local L_Icon = CreateFrame("Frame", "BGS_Icon", BGS_FrameTitle)
	BGS_Icon:SetWidth(24)
	BGS_Icon:SetHeight(24)
		FactionIcon()
	BGS_Icon:SetPoint("topleft", BGS_FrameTitle, "topleft", 0, -2)
	BGS_Icon.text = BGS_Icon:CreateFontString(nil, nil, "QuestFont_Shadow_Small")
	BGS_Icon.text:SetPoint("left", 25, 0)
	BGS_Icon.text:SetText("BG Stats")
	BGS_Icon:SetFrameLevel(L_FrameTitle:GetFrameLevel())
	BGS_Icon:Show()

-- Mainframe Close Button	
------------------------------------------------------------------------------------------------------------------------
	local L_BGS_CloseButton = CreateFrame("Button", "BGS_CloseButton", BGS_FrameTitle)
	BGS_CloseButton:SetWidth(16)
	BGS_CloseButton:SetHeight(16)
	BGS_CloseButton:SetHitRectInsets(4, 4, 4, 4)
	BGS_CloseButton:SetNormalTexture("Interface\\BUTTONS\\UI-Panel-MinimizeButton-Up")
	BGS_CloseButton:SetHighlightTexture("Interface\\BUTTONS\\UI-Panel-MinimizeButton-Highlight")
	BGS_CloseButton:SetPushedTexture("Interface\\BUTTONS\\UI-Panel-MinimizeButton-Down")
	BGS_CloseButton:SetPoint("topright", BGS_FrameTitle, "topright", 0, -1)
	BGS_CloseButton:Show()
	BGS_CloseButton:SetScript("OnClick",  function() 
		PlaySound("igQuestLogClose");
		SetShowMainframe(false) 
	end)

-- Mainframe Config Button
------------------------------------------------------------------------------------------------------------------------
	local L_BGS_ConfigButton = CreateFrame("Button", "BGS_ConfigButton", BGS_FrameTitle)
	BGS_ConfigButton:SetWidth(8)
	BGS_ConfigButton:SetHeight(8)
	BGS_ConfigButton:SetNormalTexture("Interface\\BUTTONS\\UI-GuildButton-PublicNote-Up")
	BGS_ConfigButton:SetHighlightTexture("Interface\\BUTTONS\\UI-GuildButton-PublicNote-Up")
	BGS_ConfigButton:SetPushedTexture("Interface\\BUTTONS\\UI-GuildButton-OfficerNote-Up")
	BGS_ConfigButton:SetPoint("topright", BGS_FrameTitle, "topright", -4, -15)
	BGS_ConfigButton:Show()
	BGS_ConfigButton:SetScript("OnClick",  function() 
		if ( not InterfaceOptionsFramePanelContainer.displayedPanel ) then
			InterfaceOptionsFrame_OpenToCategory(CONTROLS_LABEL);
		end
		InterfaceOptionsFrame_OpenToCategory(addonName)
	end)

-- Mainframe Move Button
------------------------------------------------------------------------------------------------------------------------
	local L_BGS_MoveButton = CreateFrame("Button", "BGS_MoveButton", BGS_FrameTitle)
	BGS_MoveButton:SetWidth(8)
	BGS_MoveButton:SetHeight(8)
	BGS_MoveButton.tex = BGS_MoveButton:CreateTexture("BGS_MoveButton_Tex")
	BGS_MoveButton.tex:SetTexture("Interface\\COMMON\\UI-ModelControlPanel")
	BGS_MoveButton.tex:SetPoint("topright", BGS_FrameTitle, "topright", -13, -15)
	BGS_MoveButton.tex:SetTexCoord(18/64, 36/64, 37/128, 53/128)
	BGS_MoveButton.tex:SetSize(8,8)
	BGS_MoveButton.tex:SetVertexColor(.8, .8, .8 ) 
	BGS_MoveButton:SetPoint("topright", BGS_FrameTitle, "topright", -13, -15)
	BGS_MoveButton:Show()
	BGS_MoveButton:SetScript("OnClick",  function() 
		ToggleLockbutton()
	end)
	BGS_MoveButton:SetScript("OnEnter",  function() 
		BGS_MoveButton.tex:SetVertexColor(1, 1, 1 )
	end)
	BGS_MoveButton:SetScript("OnLeave",  function() 
		if BGStatFrame:IsMouseEnabled() then
			BGS_MoveButton.tex:SetVertexColor(DEFAULT_LOCKVERTEX_ON, DEFAULT_LOCKVERTEX_ON, DEFAULT_LOCKVERTEX_ON )
		else	
			BGS_MoveButton.tex:SetVertexColor(DEFAULT_LOCKVERTEX_OFF, DEFAULT_LOCKVERTEX_OFF, DEFAULT_LOCKVERTEX_OFF )
		end
	end)

end

----------------------------------------
-- Small Frames
----------------------------------------
local function _CreateSmallFrames()
	-- Create a tracking frame for every tracking option
	for k, v in ipairs(BGS_FrameInfoTable) do
		local frame = CreateFrame("frame", "BGS_"..v.name, BGStatFrame)
		-- text
		frame.text = frame:CreateFontString(nil, nil, "GameFontNormal")
		frame.text:SetPoint("left", DEFAULT_TRACKOFFSET_WITH, 0)
		frame.text:SetJustifyH("left");
		frame.text:SetWordWrap(false)
		-- icon
		frame.icon = frame:CreateTexture(v.name.."_Icon")
		frame.icon:SetTexture(v.icon)
		frame.icon:SetPoint("left", frame, "left", iconOffsetX, iconOffsetY)
		frame.icon:SetSize(18,18) 
		frame.icon:Show()
		frame.detail = v.detail
		frame.info = v.info
		frame.options = v.options
		frame.visiPos = 0
		frame:SetPoint("topleft", BGStatFrame, "topleft", 0, -25)
		frame:SetWidth(BGStatFrame:GetWidth())
		frame.text:SetWidth(BGStatFrame:GetWidth()-20)
		frame:SetHeight(20)
		frame:Hide()
		table.insert(DefaultFrames, k, frame)
	end
end



----------------------------------------
-- Functions
----------------------------------------

local function ChangeBG(bg)
	if bg == "none" then
		ActiveBG = nil
	else
		ActiveBG = bg
	end
	mainBGBack = ActiveBG
	UpdateMainFrameBG()
end

local function TrackFramePos()
	local spacing = -6
	if BGS_FrameTitle:IsShown() then
		spacing = spacing - 20
	end
	
	-- create array with only shown frames ordered by visipos
	local shownArr = {}
	for k, v in ipairs(DefaultFrames) do
		if v.visiPos > 0 then
			shownArr[v.visiPos] = v
		end
	end
	
	-- place frames at their position
	for  id, frame in ipairs(shownArr) do
		local TempFrame = frame
		TempFrame:SetPoint("topleft", BGStatFrame, "topleft", 0, spacing)
		spacing = spacing - TempFrame:GetHeight()
		TempFrame:Show()
	end
end

local function ResizeBG()
	local BGSize = 14
	if BGS_FrameTitle:IsShown() then
		BGSize = BGSize + 19
	end
	for  id, frame in ipairs(DefaultFrames) do
		if frame.visiPos > 0 then
			BGSize = BGSize + frame:GetHeight()
		end
	end
	BGStatFrame:SetHeight(BGSize)
	return true
end

local function DoShowFrames()
-- Show every frame and their icons in chosen.
	for k, v in ipairs(DefaultFrames) do
		v:Show()
		if L_BGS_CBHideIcons:GetValue() then
			v.icon:Show()
		end
	end
end

local function DoHiddenFrames()
-- Hide All frames that shouldn't be shown
	for k, v in ipairs(DefaultFrames) do
		if v.visiPos == 0 then
			v:Hide()
			v.icon:Hide()
		end
	end
end

local function Options_PlaceHiddenFrames()
	local ypos = 5
	local step = 20
	local count = 1
	for k1, v1 in ipairs(DefaultFrames) do -- hidden compare to default to keep the same order
		if v1.visiPos == 0 then
			
			local frame = GetOptionsFrameByInfo(v1.detail)
			if frame == -1 then
				return
			end

			local btnUp, btnDown, move = frame:GetChildren()
			btnDown:Hide()
			btnUp:Hide()
			move:SetNormalTexture("Interface\\BUTTONS\\UI-SpellbookIcon-NextPage-Up")
			move:SetHighlightTexture("Interface\\BUTTONS\\UI-SpellbookIcon-NextPage-Down")
			move:SetPushedTexture("Interface\\BUTTONS\\UI-SpellbookIcon-NextPage-Down")
			-- swap to new list
			frame:SetParent(BGS_FramesortBG_Left)
			frame:SetPoint("topleft", frame:GetParent(), "topleft", 5, -ypos)
			-- color every other frame
			if (count%2 ~= 0) then
				frame.bg:SetTexture([[Interface\DialogFrame\UI-DialogBox-Background-Dark]])
				frame.bg:SetAlpha(0.8);
			else
				frame.bg:SetTexture([[Interface\CHATFRAME\CHATFRAMEBACKGROUND]])
				frame.bg:SetAlpha(0.8);
			end
			ypos = ypos + step
			count = count + 1
		end
	end
end

local function Options_PlaceShownFrames()
	local ypos = 5
	local step = 20
	-- place the shown frames in order
	local positionArr = {}
	for k, v in ipairs(DefaultFrames) do	
		if v.visiPos > 0 then
			local frame = GetOptionsFrameByInfo(v.detail)
			if frame == -1 then
				return
			end
			positionArr[v.visiPos] = frame
		end
	end
	
	for k, v in ipairs(positionArr) do
		local btnUp, btnDown, move = v:GetChildren()
		btnDown:Show()
		btnUp:Show()
		move:SetNormalTexture("Interface\\BUTTONS\\UI-SpellbookIcon-PrevPage-Up")
		move:SetHighlightTexture("Interface\\BUTTONS\\UI-SpellbookIcon-PrevPage-Down")
		move:SetPushedTexture("Interface\\BUTTONS\\UI-SpellbookIcon-PrevPage-Down")
		-- swap to new list
		v:SetParent(BGS_FramesortBG_Right)
		v:SetPoint("topleft", v:GetParent(), "topleft", 6, -ypos)
		v:SetPoint("left", 0, 0)
		v:SetPoint("right", -5, 0)
		-- Color every other frame
		if (k%2 ~= 0) then
			v.bg:SetTexture([[Interface\DialogFrame\UI-DialogBox-Background-Dark]])
			v.bg:SetAlpha(0.8);
		else
			v.bg:SetTexture([[Interface\CHATFRAME\CHATFRAMEBACKGROUND]])
			v.bg:SetAlpha(0.8);
		end
		ypos = ypos + step
		if k == 1 then
			btnUp:Hide()
		elseif k == table.getn(positionArr) then
			btnDown:Hide()
		end
	end
	
end

local function PlaceOptionFrames()
	 Options_PlaceShownFrames()
	 Options_PlaceHiddenFrames()
end

local function ShowFrame(key)
	local frame = DefaultFrames[key]
	if (frame) then
		-- Get current highest pos and add the current frame after than
		local highestPos = 0
		for k, v in ipairs(DefaultFrames) do
			if v.visiPos > highestPos then
				highestPos = v.visiPos
			end
		end
		frame.visiPos = highestPos + 1
	end
end

local function HideFrame(key)
	local frame = DefaultFrames[key]
	if (frame) then
		-- change pos of hidden to 0
		local removedPos = frame.visiPos
		frame.visiPos = 0
		-- move frames after hidden one 1 position down.
		for k, v in ipairs(DefaultFrames) do
			if v.visiPos > removedPos then
				v.visiPos = v.visiPos - 1
			end
		end
	end
end

local function GetPositionInDefault(id)
	for k, v in ipairs(DefaultFrames) do	
		if v == id then
			return k
		end
	end
	return -1
end

local function GetPositionInDefaultName(s)
	for k, v in ipairs(DefaultFrames) do	
		if v[2] == s then
			return k
		end
	end
	return -1
end

local function GetPositionInShown(key)
	for k, v in ipairs(ShownFrames) do	
		if v == DefaultFrames[key] then
			return k
		end
	end
	return 0
end

local function MoveUpInShown(key)
	-- Find other frame to switch places
	local temp = DefaultFrames[key].visiPos
	local otherFrame
	for k, v in ipairs(DefaultFrames) do
		if v.visiPos == temp -1 then
			otherFrame = v
		end
	end
	local pos = otherFrame.visiPos 
	otherFrame.visiPos = DefaultFrames[key].visiPos
	DefaultFrames[key].visiPos = pos

end

local function MoveDownInShown(key)
	-- Find other frame to switch places
	local temp = DefaultFrames[key].visiPos
	local otherFrame
	for k, v in ipairs(DefaultFrames) do
		if v.visiPos == temp +1 then
			otherFrame = v
		end
	end
	local pos = otherFrame.visiPos 
	otherFrame.visiPos = DefaultFrames[key].visiPos
	DefaultFrames[key].visiPos = pos
end

local function FixOptionFrames()
	DoShowFrames()
	DoHiddenFrames()
	Options_PlaceHiddenFrames()
	Options_PlaceShownFrames()
	TrackFramePos()
	ResizeBG()
end


----------------------------------------
-- Options Frame
----------------------------------------
local function Options_ChangeListstHeights()
	local step = 20
	local size = 10 + table.getn(DefaultFrames) * step
	if size < 110 then
		size = 110
	end
	BGS_FramesortBG_Left:SetHeight(size)
	BGS_FramesortBG_Left.bg:SetHeight(size-12)
end

local function _CreateOptionsFrames()

	local ypos = 16
	local step = 20
	
-- Base Frame
------------------------------------------------------------------------------------------------------------------------
	local L_BGS_Option = CreateFrame("Frame", "BGS_Option", InterfaceOptionsFramePanelContainer)
	L_BGS_Option:SetScript("OnShow",  function() 
		_UpdateWinrate()	
	end)
	BGS_Option:Hide()
	BGS_Option:SetPoint("topleft", 0, 0)
	BGS_Option:SetPoint("bottomright", 0, 0)
	BGS_Option.name = addonName
	
-- Options Title
------------------------------------------------------------------------------------------------------------------------
	local BGS_Option_Title = BGS_Option:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	BGS_Option_Title:SetPoint("TOPLEFT", 16, -ypos)
	BGS_Option_Title:SetText(addonName .. " v" .. versionNr)

-- Move button
------------------------------------------------------------------------------------------------------------------------
	local L_BGS_LockButton = CreateFrame("Button", "BGS_LockButton", BGS_Option)
	BGS_LockButton:SetWidth(16)
	BGS_LockButton:SetHeight(16)
	BGS_LockButton:SetPoint("TOPLEFT", BGS_Option, "TOPLEFT", 140, -ypos)
	BGS_LockButton.text = BGS_LockButton:CreateFontString(nil, nil, "GameFontNormal")
	BGS_LockButton.text:SetPoint("top", 0, -3)
	BGS_LockButton:Show()
	BGS_LockButton:SetScript("ONCLICK", ToggleLockbutton)
	BGS_LockButton.tex = BGS_LockButton:CreateTexture("BGS_LockButton_Tex")
	BGS_LockButton.tex:SetTexture("Interface\\COMMON\\UI-ModelControlPanel")
	BGS_LockButton.tex:SetPoint("center", BGS_LockButton, "center", 0, 0)
	BGS_LockButton.tex:SetTexCoord(18/64, 36/64, 37/128, 53/128)
	BGS_LockButton.tex:SetSize(17,17)
	BGS_LockButton.tex:SetVertexColor(DEFAULT_LOCKVERTEX_ON, DEFAULT_LOCKVERTEX_ON, DEFAULT_LOCKVERTEX_ON ) 
	BGS_LockButton:SetScript("OnEnter",  function() 
		BGS_LockButton.tex:SetVertexColor(1, 1, 1 )
		Text_OptionInfo.text:SetText("Enable/disable frame movement")
	end)
	BGS_LockButton:SetScript("OnLeave",  function() 
		if BGStatFrame:IsMouseEnabled() then
			BGS_LockButton.tex:SetVertexColor(DEFAULT_LOCKVERTEX_ON, DEFAULT_LOCKVERTEX_ON, DEFAULT_LOCKVERTEX_ON )
		else	
			BGS_LockButton.tex:SetVertexColor(DEFAULT_LOCKVERTEX_OFF, DEFAULT_LOCKVERTEX_OFF, DEFAULT_LOCKVERTEX_OFF )
		end
		Text_OptionInfo.text:SetText("")
	end)

-- Show/Hide Button
------------------------------------------------------------------------------------------------------------------------
	local L_BGS_VisibleButton = CreateFrame("Button", "BGS_VisibleButton", BGS_Option)
	BGS_VisibleButton:SetWidth(16)
	BGS_VisibleButton:SetHeight(16)
	BGS_VisibleButton:SetPoint("TOPLEFT", BGS_Option, "TOPLEFT", 160, -ypos)
	BGS_VisibleButton.text = BGS_VisibleButton:CreateFontString(nil, nil, "GameFontNormal")
	BGS_VisibleButton.text:SetPoint("top", 0, -3)
	BGS_VisibleButton:Show()
	BGS_VisibleButton:SetScript("ONCLICK", function() SetShowMainframe((not BGStatFrame:IsShown())) end)
	BGS_VisibleButton.tex = BGS_VisibleButton:CreateTexture("BGS_VisibleButton_Tex")
	BGS_VisibleButton.tex:SetTexture("Interface\\LFGFRAME\\BattlenetWorking9")
	BGS_VisibleButton.tex:SetPoint("center", BGS_VisibleButton, "center", 0, 0)
	BGS_VisibleButton.tex:SetSize(25,25)
	BGS_VisibleButton:SetScript("OnEnter",  function() 
		BGS_VisibleButton.tex:SetTexture("Interface\\LFGFRAME\\BattlenetWorking2")
		Text_OptionInfo.text:SetText("Show/hide tracking frame.")
	end)
	BGS_VisibleButton:SetScript("OnLeave",  function() 
		if BGStatFrame:IsShown() then
			BGS_VisibleButton.tex:SetTexture("Interface\\LFGFRAME\\BattlenetWorking9")
		else	
			BGS_VisibleButton.tex:SetTexture("Interface\\LFGFRAME\\BattlenetWorking4")
		end
		Text_OptionInfo.text:SetText("")
	end)

-- Button Mouseover Text
------------------------------------------------------------------------------------------------------------------------
	local L_Text_OptionInfo = CreateFrame("FRAME", "Text_OptionInfo", BGS_Option)
	Text_OptionInfo:SetHeight(15);
	Text_OptionInfo:SetWidth(100);
	Text_OptionInfo:SetPoint("topleft", BGS_Option, "topleft", 200, -ypos)
	Text_OptionInfo.text = Text_OptionInfo:CreateFontString(nil, nil, FontCheckBox)
	Text_OptionInfo.text:SetPoint("left", 0, 0)
	Text_OptionInfo.text:SetText("")
	Text_OptionInfo:Show()

-- Main Scrollcontainer
------------------------------------------------------------------------------------------------------------------------
	local Options_MainScrollContainer = AceGUI:Create("SimpleGroup") -- "InlineGroup" is also good
	Options_MainScrollContainer:SetFullWidth(true)
	Options_MainScrollContainer:SetWidth(185)
	Options_MainScrollContainer:SetFullHeight(true) -- probably?
	Options_MainScrollContainer:SetLayout("Fill") -- important!
	Options_MainScrollContainer.frame:SetParent(BGS_Option)
	Options_MainScrollContainer.frame:SetPoint("topleft", 20, -40)
	Options_MainScrollContainer.frame:SetPoint("bottomright", -5, 5)

	local Options_MainScroller = AceGUI:Create("ScrollFrame")
	Options_MainScroller:SetLayout("Flow") -- probably?
	Options_MainScrollContainer:AddChild(Options_MainScroller)

-- Options List Info
------------------------------------------------------------------------------------------------------------------------
	local L_BGS_Txt_ListInfo = AceGUI:Create("SimpleGroup")
	L_BGS_Txt_ListInfo:SetFullWidth(true)
	L_BGS_Txt_ListInfo:SetHeight(35)
	L_BGS_Txt_ListInfo:SetLayout("Fill")
	L_BGS_Txt_ListInfo.text = L_BGS_Txt_ListInfo.frame:CreateFontString(nil, nil, "GameFontHighlightSmall")
	L_BGS_Txt_ListInfo.text:SetHeight(L_BGS_Txt_ListInfo.frame:GetHeight())
	L_BGS_Txt_ListInfo.text:SetText("You can show or hide the different frames by moving them between lists using the buttons \n You can also change the order of the visible frames.")
	L_BGS_Txt_ListInfo.text:SetPoint("topleft", 0, 0)
	L_BGS_Txt_ListInfo.text:SetJustifyH("left", 0, 0)
	L_BGS_Txt_ListInfo.text:SetPoint("topright", 0, 0)
	Options_MainScroller:AddChild(L_BGS_Txt_ListInfo)

-- Options Lists Container
------------------------------------------------------------------------------------------------------------------------
	local optionframeWidth = 182

	local L_BGS_ListsGroup = AceGUI:Create("SimpleGroup")
	L_BGS_ListsGroup:SetFullWidth(true)
	L_BGS_ListsGroup:SetHeight(35)
	L_BGS_ListsGroup:SetLayout("Fill")
	Options_MainScroller:AddChild(L_BGS_ListsGroup)

-- Hidden Frame Box
------------------------------------------------------------------------------------------------------------------------
	local L_BGS_FramesortBG_Left = CreateFrame("frame", "BGS_FramesortBG_Left", L_BGS_ListsGroup.frame)
	BGS_FramesortBG_Left:SetPoint("topleft", L_BGS_ListsGroup.frame, "topleft", 0, -10)
	BGS_FramesortBG_Left:SetBackdrop({bgFile = nill,
		edgeFile = DEFAULT_EDGEFILE, tile = true,
		tileSize = 300, edgeSize = 16,
		insets = { left = 3, right = 3, top = 3, bottom = 3 }
	})
	BGS_FramesortBG_Left:SetWidth(150)
	BGS_FramesortBG_Left:SetHeight(175)  
	BGS_FramesortBG_Left.bg = BGS_FramesortBG_Left:CreateTexture("BGS_FramesortBG_Left_BG")
	BGS_FramesortBG_Left.bg:SetTexture([[Interface\FriendsFrame\UI-FriendsFrame-HighlightBar]])
	BGS_FramesortBG_Left.bg:SetVertexColor(0, 0, 0)
	BGS_FramesortBG_Left.bg:SetAlpha(0.6)
	BGS_FramesortBG_Left.bg:SetPoint("center", BGS_FramesortBG_Left, "center", 0, 0)
	BGS_FramesortBG_Left.bg:SetSize(BGS_FramesortBG_Left:GetWidth()-6,BGS_FramesortBG_Left:GetHeight()-6) 
	BGS_FramesortBG_Left.bg:Show()
	BGS_FramesortBG_Left.label = BGS_FramesortBG_Left:CreateTexture("BGS_MoveButton_Tex")
	BGS_FramesortBG_Left.label:SetTexture("Interface\\COMMON\\Common-Input-Border")
	BGS_FramesortBG_Left.label:SetTexCoord(0, 1, 0, 0.5)
	BGS_FramesortBG_Left.label:SetPoint("top", BGS_FramesortBG_Left, "top", 0, 15)
	BGS_FramesortBG_Left.text = BGS_FramesortBG_Left:CreateFontString(nil, nil, FontCheckBox)
	BGS_FramesortBG_Left.text:SetPoint("top", 0, 12)
	BGS_FramesortBG_Left.text:SetText("Hidden Frames") 
	BGS_FramesortBG_Left.label:SetSize(BGS_FramesortBG_Left:GetWidth()-20, 18)
	Options_ChangeListstHeights()
	
	L_BGS_ListsGroup:SetHeight(BGS_FramesortBG_Left:GetHeight() + 20)

-- Visible Frame Box
------------------------------------------------------------------------------------------------------------------------
	local L_BGS_FramesortBG_Right = CreateFrame("frame", "BGS_FramesortBG_Right", L_BGS_ListsGroup.frame)
	BGS_FramesortBG_Right:SetPoint("topleft", L_BGS_ListsGroup.frame, "topleft", 170, -10)
	BGS_FramesortBG_Right:SetBackdrop({bgFile = nill,
		edgeFile = DEFAULT_EDGEFILE, tile = true,
		tileSize = 200, edgeSize = 16,
		insets = { left = 3, right = 3, top = 3, bottom = 3 }
	})
	BGS_FramesortBG_Right:SetWidth(optionframeWidth)
	BGS_FramesortBG_Right:SetHeight(BGS_FramesortBG_Left:GetHeight())
	BGS_FramesortBG_Right.bg = BGS_FramesortBG_Right:CreateTexture("BGS_FramesortBG_Right_BG")
	BGS_FramesortBG_Right.bg:SetTexture([[Interface\FriendsFrame\UI-FriendsFrame-HighlightBar]])
	BGS_FramesortBG_Right.bg:SetVertexColor(0, 0, 0)
	BGS_FramesortBG_Right.bg:SetAlpha(0.6)
	BGS_FramesortBG_Right.bg:SetPoint("center", BGS_FramesortBG_Right, "center", 0, 0)
	BGS_FramesortBG_Right.bg:SetSize(BGS_FramesortBG_Right:GetWidth()-12,BGS_FramesortBG_Right:GetHeight()-12) 
	BGS_FramesortBG_Right.bg:Show()
	BGS_FramesortBG_Right.label = BGS_FramesortBG_Right:CreateTexture("BGS_MoveButton_Tex")
	BGS_FramesortBG_Right.label:SetTexture("Interface\\COMMON\\Common-Input-Border")
	BGS_FramesortBG_Right.label:SetTexCoord(0, 1, 0, 0.5)
	BGS_FramesortBG_Right.label:SetPoint("top", BGS_FramesortBG_Right, "top", 0, 15)
	BGS_FramesortBG_Right.text = BGS_FramesortBG_Right:CreateFontString(nil, nil, FontCheckBox)
	BGS_FramesortBG_Right.text:SetPoint("top", 0, 12)
	BGS_FramesortBG_Right.text:SetText("Visible Frames")
	BGS_FramesortBG_Right.label:SetSize(BGS_FramesortBG_Right:GetWidth()-20, 18)

-- Frame Info Box
------------------------------------------------------------------------------------------------------------------------
	local L_BGS_FramesortBG_Info = CreateFrame("frame", "BGS_FramesortBG_Info", L_BGS_ListsGroup.frame)
	BGS_FramesortBG_Info:SetPoint("topleft", L_BGS_ListsGroup.frame, "topleft", 170 + BGS_FramesortBG_Right:GetWidth() + 10, -10)
	BGS_FramesortBG_Info:SetBackdrop({bgFile = nill,
		edgeFile = DEFAULT_EDGEFILE, tile = true,
		tileSize = 200, edgeSize = 16,
		insets = { left = 3, right = 3, top = 3, bottom = 3 }
	})
	BGS_FramesortBG_Info:SetWidth(200)
	BGS_FramesortBG_Info:SetHeight(BGS_FramesortBG_Left:GetHeight())
	BGS_FramesortBG_Info.bg = BGS_FramesortBG_Info:CreateTexture("BGS_FramesortBG_Info_BG")
	BGS_FramesortBG_Info.bg:SetTexture([[Interface\FriendsFrame\UI-FriendsFrame-HighlightBar]])
	BGS_FramesortBG_Info.bg:SetVertexColor(0, 0, 0)
	BGS_FramesortBG_Info.bg:SetAlpha(0.6)
	BGS_FramesortBG_Info.bg:SetPoint("center", BGS_FramesortBG_Info, "center", 0, 0)
	BGS_FramesortBG_Info.bg:SetSize(BGS_FramesortBG_Info:GetWidth()-12,BGS_FramesortBG_Info:GetHeight()-12) 
	BGS_FramesortBG_Info.bg:Show()
	BGS_FramesortBG_Info.label = BGS_FramesortBG_Info:CreateTexture("BGS_MoveButton_Tex")
	BGS_FramesortBG_Info.label:SetTexture("Interface\\COMMON\\Common-Input-Border")
	BGS_FramesortBG_Info.label:SetTexCoord(0, 1, 0, 0.5)
	BGS_FramesortBG_Info.label:SetPoint("top", BGS_FramesortBG_Info, "top", 0, 15)
	BGS_FramesortBG_Info.text = BGS_FramesortBG_Info:CreateFontString(nil, nil, FontCheckBox)
	BGS_FramesortBG_Info.text:SetPoint("top", 0, 12)
	BGS_FramesortBG_Info.text:SetText("Track Options")
	BGS_FramesortBG_Info.label:SetSize(BGS_FramesortBG_Info:GetWidth()-20, 18)

-- Main Scrollcontainer
------------------------------------------------------------------------------------------------------------------------
	local Options_FrameInfoDefault = AceGUI:Create("SimpleGroup")
	Options_FrameInfoDefault:SetFullWidth(true)
	Options_FrameInfoDefault:SetHeight(35)
	Options_FrameInfoDefault:SetLayout("Fill")
	Options_FrameInfoDefault.bg = Options_FrameInfoDefault.frame:CreateTexture("blmavla")
	Options_FrameInfoDefault.bg:SetTexture("Interface\\LFGFRAME\\UI-LFG-SEPARATOR")
	Options_FrameInfoDefault.bg:SetHeight(Options_FrameInfoDefault.frame:GetHeight()/2)
	Options_FrameInfoDefault.bg:SetTexCoord(0, 168/256, 0, 34/128)
	Options_FrameInfoDefault.bg:SetPoint("bottom", 0, -2)
	Options_FrameInfoDefault.text = Options_FrameInfoDefault.frame:CreateFontString(nil, nil, "GameFontHighlightSmall")
	Options_FrameInfoDefault.text:SetHeight(Options_FrameInfoDefault.frame:GetHeight())
	Options_FrameInfoDefault.text:SetText("Click a frame on the left for more info and options.")
	Options_FrameInfoDefault.text:SetPoint("topleft", 0, 0)
	Options_FrameInfoDefault.text:SetPoint("topright", 0, 0)
	Options_FrameInfoDefault.frame:SetPoint("top", BGS_FramesortBG_Info, "top", 0, -10)
	Options_FrameInfoDefault.frame:SetParent(BGS_FramesortBG_Info)
	Options_FrameInfoDefault:SetWidth(BGS_FramesortBG_Info:GetWidth()-20)
	Options_FrameInfoDefault.frame:Show()


-- Create small Frame Options
------------------------------------------------------------------------------------------------------------------------
	for k,v in  ipairs(DefaultFrames) do
		local trackName = "BGS_Optframe_"..v.detail
		-- Create a small frame
		local BGS_OptframeDefault = CreateFrame("Button", trackName, BGS_FramesortBG_Left)
		BGS_OptframeDefault:SetPoint("topleft", BGS_OptframeDefault:GetParent(), "topleft", 5, 0)
		BGS_OptframeDefault:SetWidth(BGS_OptframeDefault:GetParent():GetWidth()-10)
		BGS_OptframeDefault:SetHeight(20)
		BGS_OptframeDefault:SetPoint("left", 0, 0)
		BGS_OptframeDefault:SetPoint("right", -5, 0)
		-- Default Background
		BGS_OptframeDefault.bg = BGS_OptframeDefault:CreateTexture(trackName.."_BG")
		BGS_OptframeDefault.bg:SetTexture([[Interface\CHATFRAME\CHATFRAMEBACKGROUND]])
		BGS_OptframeDefault.bg:SetDrawLayer("background", 0)
		BGS_OptframeDefault.bg:SetParent(BGS_OptframeDefault)
		BGS_OptframeDefault.bg:SetPoint("topleft", 0, 0)
		BGS_OptframeDefault.bg:SetPoint("bottomright", 0, 0)
		BGS_OptframeDefault.bg:SetVertexColor(.2, .2, .2)
		BGS_OptframeDefault.bg:SetAlpha(0.6)
		BGS_OptframeDefault.bg:SetPoint("left", BGS_OptframeDefault, "left", 0, 0)
		BGS_OptframeDefault.bg:SetSize(BGS_OptframeDefault:GetWidth(),BGS_OptframeDefault:GetHeight()) 
		BGS_OptframeDefault.bg:Show()
		-- Selected frame Background
		BGS_OptframeDefault.hl = BGS_OptframeDefault:CreateTexture(trackName.."_Highlight")
		BGS_OptframeDefault.hl:SetTexture(nil)
		BGS_OptframeDefault.hl:SetDrawLayer("background", 1)
		BGS_OptframeDefault.hl:SetAlpha(1)
		BGS_OptframeDefault.hl:SetParent(BGS_OptframeDefault)
		BGS_OptframeDefault.hl:SetPoint("topleft", 0, 0)
		BGS_OptframeDefault.hl:SetPoint("bottomright", 0, 0)
		BGS_OptframeDefault.hl:SetSize(BGS_OptframeDefault:GetWidth(),BGS_OptframeDefault:GetHeight()) 
		BGS_OptframeDefault.hl:Show()
		--BGS_OptframeDefault:SetWidth(BGS_OptframeDefault:GetParent():GetWidth()-10)
		BGS_OptframeDefault:SetHighlightTexture([[Interface\FriendsFrame\UI-FriendsFrame-HighlightBar-Blue]])
		-- Face Text
		BGS_OptframeDefault.text = BGS_OptframeDefault:CreateFontString(nil, nil, "GameFontNormal")
		BGS_OptframeDefault.text:SetPoint("left", 5, 0)
		BGS_OptframeDefault.text:SetText(v.detail)
		BGS_OptframeDefault:SetScript("OnClick",  function(self)
			PlaySound("igMainMenuOptionCheckBoxOn");
			local info = v.info
			FixOptionFrames()
			-- Reset all frames
			for k, v in ipairs(OptionsFrame) do
				v.bg:SetVertexColor(.2, .2, .2)
				v.bg:SetTexCoord(0,1,0,1)
				v.hl:SetTexture(nil)
			end
			
			-- Give highlight to clicked frame
			BGS_OptframeDefault.hl:SetTexture([[Interface\FriendsFrame\UI-FriendsFrame-HighlightBar]])
			-- Change label of info box
			BGS_FramesortBG_Info.text:SetText(BGS_OptframeDefault.text:GetText())
			
			-- Hide all children in info box
			if {BGS_FramesortBG_Info:GetChildren()} ~= nill	then
				for k, child in ipairs({BGS_FramesortBG_Info:GetChildren()}) do
					child:Hide()
				end
			end
			
			-- Place the info/options of the frame in the info box
			if v.options ~= nill then
				for k, f in ipairs(v.options) do
					f.frame:SetPoint("right", BGS_FramesortBG_Info, "right", -6, 0)
					f.frame:SetParent(BGS_FramesortBG_Info)
					f:SetWidth(BGS_FramesortBG_Info:GetWidth()-15)
					f:SetHeight(BGS_FramesortBG_Info:GetHeight()-12)
					f.frame:Show()
				end
			end
			
		end)-- end of frame onclick function

		table.insert(OptionsFrame, BGS_OptframeDefault)

		-- create up button
		local moveUp = CreateFrame("Button", trackName.."_moveUp", BGS_OptframeDefault)
		moveUp:SetWidth(OPTIONS_TINYBTN_SIZE)
		moveUp:SetHeight(OPTIONS_TINYBTN_SIZE)
		moveUp:SetNormalTexture("Interface\\CHATFRAME\\UI-ChatIcon-ScrollUp-Up")
		moveUp:SetHighlightTexture("Interface\\CHATFRAME\\UI-ChatIcon-ScrollUp-Down")
		moveUp:SetPushedTexture("Interface\\CHATFRAME\\UI-ChatIcon-ScrollUp-Down")
		moveUp:SetPoint("left", trackName, "left", 122 +(15+1)*1, 0)
		moveUp:SetScript("OnClick",  function(self)
			PlaySound("igMainMenuOptionCheckBoxOn");
			MoveUpInShown(k)
			FixOptionFrames()
		end)
		moveUp:Hide()

		-- create down button
		local moveDown = CreateFrame("Button", trackName.."_moveDown", BGS_OptframeDefault)
		moveDown:SetWidth(OPTIONS_TINYBTN_SIZE)
		moveDown:SetHeight(OPTIONS_TINYBTN_SIZE)
		moveDown:SetNormalTexture("Interface\\CHATFRAME\\UI-ChatIcon-ScrollDown-Up")
		moveDown:SetHighlightTexture("Interface\\CHATFRAME\\UI-ChatIcon-ScrollDown-Down")
		moveDown:SetPushedTexture("Interface\\CHATFRAME\\UI-ChatIcon-ScrollDown-Down")
		moveDown:SetPoint("left", trackName, "left", 122 +(15+1)*2, 0)
		moveDown:SetScript("OnClick",  function(self)
			PlaySound("igMainMenuOptionCheckBoxOn");
			MoveDownInShown(k)
			FixOptionFrames()
		end)
		moveDown:Hide()

		-- create move button 
		local move = CreateFrame("Button", trackName.."_move", BGS_OptframeDefault)
		move:SetWidth(OPTIONS_TINYBTN_SIZE)
		move:SetHeight(OPTIONS_TINYBTN_SIZE)
		move:SetNormalTexture("Interface\\BUTTONS\\UI-SpellbookIcon-NextPage-Up")
		move:SetHighlightTexture("Interface\\BUTTONS\\UI-SpellbookIcon-NextPage-Down")
		move:SetPushedTexture("Interface\\BUTTONS\\UI-SpellbookIcon-NextPage-Down")
		move:SetPoint("left", trackName, "left", 122, 0)
		move:Show()
		move:SetScript("OnClick",  function(self)
			PlaySound("igMainMenuOptionCheckBoxOn");
			local parent = self:GetParent()
			local btnUp, btnDown = parent:GetChildren()
			-- swap frame between boxes
			if parent:GetParent():GetName() == "BGS_FramesortBG_Right" then
				parent:SetParent(BGS_FramesortBG_Left)
				HideFrame(k)
				btnUp:Hide()
				btnDown:Hide()
			else
				parent:SetParent(BGS_FramesortBG_Right)
				btnUp:Show()
				btnDown:Show()
				ShowFrame(k)
			end
			--parent:SetPoint("topleft", parent:GetParent(), "topleft", 5, -5)
			--parent:SetWidth(parent:GetParent():GetWidth()-10)
			parent:SetPoint("left", 0, 0)
			parent:SetPoint("right", -5, 0)
			FixOptionFrames()
		end)
		
end -- end of for loop


-- Background Dropdown
------------------------------------------------------------------------------------------------------------------------
	local ddwn_Background_Container = AceGUI:Create("SimpleGroup")
	ddwn_Background_Container:SetRelativeWidth(0.6)
	Options_MainScroller:AddChild(ddwn_Background_Container)
	
	ddwn_Background = AceGUI:Create("Dropdown")
	ddwn_Background:SetWidth(150)
	ddwn_Background:SetList(BGTable, BGOrderTable)
	ddwn_Background:SetLabel("Background")
	ddwn_Background:SetCallback("OnValueChanged", function(_,_, choise)
		ChangeBG(choise)
	end)
	ddwn_Background_Container:AddChild(ddwn_Background)

-- Hide Icons Checkbox
------------------------------------------------------------------------------------------------------------------------
	L_BGS_CBHideIcons = AceGUI:Create("CheckBox")
	--L_BGS_CBHideIcons:SetFullWidth(true)
	L_BGS_CBHideIcons:SetRelativeWidth(0.6)
	L_BGS_CBHideIcons:SetLabel("Show Icons")
	L_BGS_CBHideIcons:SetCallback("OnValueChanged", function(value)
		if L_BGS_CBHideIcons:GetValue() then
			PlaySound("igMainMenuOptionCheckBoxOn");
			for k, v in ipairs(DefaultFrames) do 
				v.text:SetPoint("left", DEFAULT_TRACKOFFSET_WITH, 0)
				if v.visiPos > 0 then
					v.icon:Show()
				end
			end
		else
			PlaySound("igMainMenuOptionCheckBoxOff");
			for k, v in ipairs(DefaultFrames) do 
				v.text:SetPoint("left", DEFAULT_TRACKOFFSET_WITHOUT, 0)
				v.icon:Hide()
			end
		end
	end)
	Options_MainScroller:AddChild(L_BGS_CBHideIcons)

-- Hide Title Checkbox
------------------------------------------------------------------------------------------------------------------------
	L_BGS_CBHideTitle = AceGUI:Create("CheckBox")
	--L_BGS_CBHideTitle:SetFullWidth(true)
	L_BGS_CBHideTitle:SetRelativeWidth(0.6)
	L_BGS_CBHideTitle:SetLabel("Show title and buttons")
	L_BGS_CBHideTitle:SetCallback("OnValueChanged", function(value)
		if  L_BGS_CBHideTitle:GetValue() then
			PlaySound("igMainMenuOptionCheckBoxOn");
			BGS_FrameTitle:Show()
			L_BGS_CBHideTitle:SetValue(true)
		else
			PlaySound("igMainMenuOptionCheckBoxOff");
			BGS_FrameTitle:Hide()
			L_BGS_CBHideTitle:SetValue(false)
		end
		TrackFramePos()
		ResizeBG()
	end)
	Options_MainScroller:AddChild(L_BGS_CBHideTitle)

-- Slash Command Info
------------------------------------------------------------------------------------------------------------------------
	local L_Text_Commands = CreateFrame("FRAME", "Text_Commands", BGS_Option)
	local commandLines = {"|cFFFFD100BGStats commands:|r"
		,"|cFFFFD100/bgstats|r: Opens this options frame."
		,"|cFFFFD100/bgstats show|r: Shows the tracking frame."
		,"|cFFFFD100/bgstats hide|r: Hides the tracking frame."
		,"|cFFFFD100/bgstats lock|r: Toggle moveable tracking frame."}
	for k, v in ipairs(commandLines) do
		local frame = _G["L_Text_Commands.text"..k]
		frame = L_Text_Commands:CreateFontString(nil, nil, FontCheckBox)
		frame:SetPoint("topleft", 0, -13 * (k-1))
		frame:SetText(v)
		frame:SetJustifyH("left")
		frame:SetJustifyV("middle")
		frame:SetHeight(15);
		frame:SetWidth(400);
	end
	L_Text_Commands:SetHeight(15*4);
	L_Text_Commands:SetWidth(400);
	L_Text_Commands:SetPoint("bottomleft", BGS_Option, "bottomleft", 15, 13*2)
	L_Text_Commands:Show()

	
-- Frame Info Box
------------------------------------------------------------------------------------------------------------------------
	local L_BGS_WinrateBox = CreateFrame("frame", "L_BGS_WinrateBox", L_BGS_ListsGroup.frame)
	L_BGS_WinrateBox:SetPoint("top", BGS_FramesortBG_Info, "bottom", 0, -25)
	L_BGS_WinrateBox:SetBackdrop({bgFile = nill,
		edgeFile = DEFAULT_EDGEFILE, tile = true,
		tileSize = 200, edgeSize = 16,
		insets = { left = 3, right = 3, top = 3, bottom = 3 }
	})
	L_BGS_WinrateBox:SetWidth(BGS_FramesortBG_Info:GetWidth())
	L_BGS_WinrateBox:SetHeight(200)
	L_BGS_WinrateBox.bg = L_BGS_WinrateBox:CreateTexture("L_BGS_WinrateBox_BG")
	L_BGS_WinrateBox.bg:SetTexture([[Interface\FriendsFrame\UI-FriendsFrame-HighlightBar]])
	L_BGS_WinrateBox.bg:SetVertexColor(0, 0, 0)
	L_BGS_WinrateBox.bg:SetAlpha(0.6)
	L_BGS_WinrateBox.bg:SetPoint("center", L_BGS_WinrateBox, "center", 0, 0)
	L_BGS_WinrateBox.bg:SetSize(L_BGS_WinrateBox:GetWidth()-12,L_BGS_WinrateBox:GetHeight()-12) 
	L_BGS_WinrateBox.bg:Show()
	L_BGS_WinrateBox.label = L_BGS_WinrateBox:CreateTexture("BGS_MoveButton_Tex")
	L_BGS_WinrateBox.label:SetTexture("Interface\\COMMON\\Common-Input-Border")
	L_BGS_WinrateBox.label:SetTexCoord(0, 1, 0, 0.5)
	L_BGS_WinrateBox.label:SetPoint("top", L_BGS_WinrateBox, "top", 0, 15)
	L_BGS_WinrateBox.text = L_BGS_WinrateBox:CreateFontString(nil, nil, FontCheckBox)
	L_BGS_WinrateBox.text:SetPoint("top", 0, 12)
	L_BGS_WinrateBox.text:SetText("Win Rates")
	L_BGS_WinrateBox.label:SetSize(L_BGS_WinrateBox:GetWidth()-20, 18)

	local WinrateScrollContainer = AceGUI:Create("SimpleGroup") -- "InlineGroup" is also good
	WinrateScrollContainer:SetFullWidth(true)
	WinrateScrollContainer:SetFullHeight(true) -- probably?
	WinrateScrollContainer:SetLayout("Fill") -- important!
	WinrateScrollContainer.frame:SetParent(L_BGS_WinrateBox)
	WinrateScrollContainer.frame:SetPoint("topleft", 10, -6)
	WinrateScrollContainer.frame:SetPoint("bottomright", -6, 5)

	local WinrateScroller = AceGUI:Create("ScrollFrame")
	WinrateScroller:SetLayout("Flow") -- probably?
	WinrateScrollContainer:AddChild(WinrateScroller)
	
	for k, bg in ipairs(_holidayInfoList) do
		local WinrateTextContainer = AceGUI:Create("SimpleGroup")
		--WinrateTextContainer:SetFullWidth(true)
		WinrateTextContainer:SetRelativeWidth(0.8)
		WinrateTextContainer:SetLayout("Fill")
		WinrateTextContainer.text = WinrateTextContainer.frame:CreateFontString(nil, nil, FontCheckBox)
		-- blizzard bug for EotS
		--local bgName = bg.full

		WinrateTextContainer.text:SetText("|cFFFFD100".. bg.full ..":|r\n " ..BGS_GetWinrateString(bg.full))
		WinrateTextContainer:SetHeight(WinrateTextContainer.text:GetHeight()+5);
		WinrateTextContainer.text:SetJustifyH("left")
		WinrateTextContainer.text:SetPoint("topleft", 5, 0)
		bg.frame = WinrateTextContainer
		
		local WinrateIcon = AceGUI:Create("Icon")
		WinrateIcon:SetImage(bg.icon)
		WinrateIcon:SetImageSize(20, 20)
		WinrateIcon:SetWidth(20)
		WinrateScroller:AddChild(WinrateIcon)
		WinrateScroller:AddChild(WinrateTextContainer)
	end

end

local function Firstrun()
	for i=0, table.getn(DefaultFrames) do
		ShowFrame(i+1)
		
	end
	for k, v in ipairs(DefaultFrames) do
		ShowFrame(k)
		v.visiPos = k
	end
	
	-- Movable
	SetShowMainframe(true)
	BGStatFrame:EnableMouse(true)
			
	BGS_LockButton.tex:SetVertexColor(DEFAULT_LOCKVERTEX_ON, DEFAULT_LOCKVERTEX_ON, DEFAULT_LOCKVERTEX_ON )
	BGStatFrame:SetBackdrop({
		bgFile = mainBGBack,
		edgeFile = [[Interface/DialogFrame/UI-DialogBox-Border]],
		tileSize = 0, edgeSize = 16,
		insets = { left = 3, right = 3, top = 3, bottom = 3 }
	})

	ChangeBG(mainBGBack)
	ddwn_Background:SetValue(mainBGBack)

	L_BGS_CBHideIcons:SetValue(true)
	L_BGS_CBHideTitle:SetValue(true)
end

local function RenameOldSave(name)
	
	for k, v in ipairs(BGS_FrameInfoTable) do
		if v[2] == name then
			return v[1]
		end
	end
	
	return "error"
end

function BGS_FrameInfoTable:GetInfoList(name)
	for k,v in ipairs(ExtraDataList) do
		if v.frame == name then
			return v
		end
	end
	return nil
end

local function CommonRunOLD()
	if type(BackgroundId) == type({}) then
		ChangeBG(BackgroundId[2])
		ddwn_Background:SetValue(BackgroundId[2])
	end
	
	

	SetShowMainframe(IsShown)

	if CanMove == false then
		BGStatFrame:EnableMouse(false)
		BGS_LockButton.tex:SetVertexColor(DEFAULT_LOCKVERTEX_OFF, DEFAULT_LOCKVERTEX_OFF, DEFAULT_LOCKVERTEX_OFF )
		BGS_MoveButton.tex:SetVertexColor(DEFAULT_LOCKVERTEX_OFF, DEFAULT_LOCKVERTEX_OFF, DEFAULT_LOCKVERTEX_OFF )
	else
		BGStatFrame:EnableMouse(true)
		BGS_LockButton.tex:SetVertexColor(DEFAULT_LOCKVERTEX_ON, DEFAULT_LOCKVERTEX_ON, DEFAULT_LOCKVERTEX_ON )
		BGS_MoveButton.tex:SetVertexColor(DEFAULT_LOCKVERTEX_ON, DEFAULT_LOCKVERTEX_ON, DEFAULT_LOCKVERTEX_ON )
	end
	
	UpdateMainFrameBG()
	  
	
	  
	if TitleHidden ~= nil and TitleHidden == true then
		L_BGS_CBHideTitle:SetValue(false)
		BGS_FrameTitle:Hide()
	else
		L_BGS_CBHideTitle:SetValue(true)
		BGS_FrameTitle:Show()
	end
		
		
	if IconsHidden ~= nil and IconsHidden == true then
		L_BGS_CBHideIcons:SetValue(false)
		for k, v in ipairs(DefaultFrames) do 
			v.icon:Hide()
			v.text:SetPoint("left", DEFAULT_TRACKOFFSET_WITHOUT, 0)
		end	
	else
		L_BGS_CBHideIcons:SetValue(true)
		for k, v in ipairs(DefaultFrames) do 
			v.icon:Show()
			v.text:SetPoint("left", DEFAULT_TRACKOFFSET_WITH, 0)
		end
	end
	
	for  id, name in ipairs(ShownFramesTable) do 
		local f = BGS_FrameInfoTable:GetFrameByName(name)
		if f == -1 then
			f = BGS_FrameInfoTable:GetFrameByName(RenameOldSave(name))
		end
		if f == -1 then
			print("BGstats: Unknown frame error, please reload the ui (/reload) to fix this issue.")
			break;
		end
		f.visiPos = id
	end
	Options_PlaceShownFrames()
	
	--ddwn_Background:SetValue(tempSavedData.MainframeBackground)
	
	if BGstats_ExtraFrameDataList ~= nil then
	ExtraDataList = BGstats_ExtraFrameDataList
	end
end

local function CommonRun()

	local tempSavedData = BGstats_BaseDataList
	ChangeBG(tempSavedData.MainframeBackground) 
	SetShowMainframe(tempSavedData.MainframeIsShown)
	if tempSavedData.MainframeIsMoveable then
		BGStatFrame:EnableMouse(true)
		BGS_LockButton.tex:SetVertexColor(DEFAULT_LOCKVERTEX_ON, DEFAULT_LOCKVERTEX_ON, DEFAULT_LOCKVERTEX_ON )
		BGS_MoveButton.tex:SetVertexColor(DEFAULT_LOCKVERTEX_ON, DEFAULT_LOCKVERTEX_ON, DEFAULT_LOCKVERTEX_ON )
	else
		BGStatFrame:EnableMouse(false)
		BGS_LockButton.tex:SetVertexColor(DEFAULT_LOCKVERTEX_OFF, DEFAULT_LOCKVERTEX_OFF, DEFAULT_LOCKVERTEX_OFF )
		BGS_MoveButton.tex:SetVertexColor(DEFAULT_LOCKVERTEX_OFF, DEFAULT_LOCKVERTEX_OFF, DEFAULT_LOCKVERTEX_OFF )
	end
	UpdateMainFrameBG()
	if tempSavedData.TitleHidden ~= nil and tempSavedData.TitleHidden == true then
		L_BGS_CBHideTitle:SetValue(true)
		BGS_FrameTitle:Show()
	else
		L_BGS_CBHideTitle:SetValue(false)
		BGS_FrameTitle:Hide()
	end
	
	if tempSavedData.IconsHidden ~= nil and tempSavedData.IconsHidden == true then
		L_BGS_CBHideIcons:SetValue(true)
		for k, v in ipairs(DefaultFrames) do 
			v.icon:Show()
			v.text:SetPoint("left", DEFAULT_TRACKOFFSET_WITH, 0)
		end
	else
		L_BGS_CBHideIcons:SetValue(false)
		for k, v in ipairs(DefaultFrames) do 
			v.icon:Hide()
			v.text:SetPoint("left", DEFAULT_TRACKOFFSET_WITHOUT, 0)
		end	
		
	end
	
	for  id, frame in ipairs(tempSavedData.ShownFrames) do 
		local f = BGS_FrameInfoTable:GetFrameByName(frame.name)
		if f == -1 then
			f = BGS_FrameInfoTable:GetFrameByName(RenameOldSave(frame.name))
		end
		if f == -1 then
			print("BGstats: Unknown frame error, please reload the UI (/reload) to fix this issue.")
			break;
		end
		f.visiPos = frame.pos
	end
	Options_PlaceShownFrames()
	
	ddwn_Background:SetValue(tempSavedData.MainframeBackground)
	
	if BGstats_ExtraFrameDataList ~= nil then
	ExtraDataList = BGstats_ExtraFrameDataList
	
	
	end
	
end
	
local L_BGS_LoadFrame = CreateFrame("FRAME", "BGS_LoadFrame"); 
BGS_LoadFrame:RegisterEvent("ADDON_LOADED");
BGS_LoadFrame:RegisterEvent("PLAYER_LOGOUT");
BGS_LoadFrame:RegisterEvent("PLAYER_LOGIN");
BGS_LoadFrame:RegisterEvent("COMPACT_UNIT_FRAME_PROFILES_LOADED");
BGS_LoadFrame:SetScript("OnEvent", function(self, event, ...) self[event](self, ...) end)

function BGS_LoadFrame:COMPACT_UNIT_FRAME_PROFILES_LOADED(loadedAddon)
	if loadedAddon ~= addonName then return end
	self:UnregisterEvent("COMPACT_UNIT_FRAME_PROFILES_LOADED")
	
	InterfaceOptions_AddCategory(BGS_Option)

	
	self.COMPACT_UNIT_FRAME_PROFILES_LOADED = nil
end

function BGS_LoadFrame:ADDON_LOADED(loadedAddon)
	if loadedAddon ~= addonName then return end
	self:UnregisterEvent("ADDON_LOADED")
	_CreateBaseFrame()
	_CreateSmallFrames()
	_CreateOptionsFrames()
	
	ResizeBG()
	
	
	for i=0, table.getn(DefaultFrames) do
		HideFrame(i+1)
	end
	if BGstats_BaseDataList == nil then -- either no data or old data
		if firstRun == nil then --not even old data so truely first run
			Firstrun()
		else
			CommonRunOLD()
		end
	else
		CommonRun()
	end
	
	InterfaceOptions_AddCategory(BGS_Option)
	
	
	self.ADDON_LOADED = nil
end

function BGS_LoadFrame:PLAYER_LOGIN(loadedAddon)
	ResizeBG()
    TrackFramePos()
	FixOptionFrames()
	
	BGstats_ExtraFrameDataList = {}
	
end

function BGS_LoadFrame:PLAYER_LOGOUT(loadedAddon)
    
	local tempSaveData = {}
	tempSaveData.MainframeIsShown = BGStatFrame:IsShown()
	tempSaveData.MainframeIsMoveable = BGStatFrame:IsMouseEnabled()
	tempSaveData.MainframeBackground = ActiveBG
	tempSaveData.TitleHidden = L_BGS_CBHideTitle:GetValue()
	tempSaveData.IconsHidden = L_BGS_CBHideIcons:GetValue()
	
	local tempTable = {}
	for  id, frame in ipairs(DefaultFrames) do 
		if frame.visiPos > 0 then -- All frames that are currently shown 
			local tempSave = {}
			tempSave.name = string.gsub(frame:GetName(), "BGS_", "")
			tempSave.pos = frame.visiPos
			table.insert(tempTable, tempSave) -- Save framename
		end
	end
	tempSaveData.ShownFrames = tempTable
	
	BGstats_BaseDataList = tempSaveData
	
end


----------------------------------------
-- Slash Commands
----------------------------------------

SLASH_PVPSTATS1 = '/bgstats';
local function slashcmd(msg, editbox)
	if msg == 'hide' then
		SetShowMainframe(false)
	elseif msg == 'show' then
		SetShowMainframe(true)
	elseif msg == 'lock' then
		ToggleLockbutton()
		--[[
	elseif msg == 'clear' then -- temp
		local frame = BGS_FrameInfoTable:GetFrameByName("SessionFrame");
		frame.text:SetText(4 .. " : " .. 5);
	-- Debug
   
   elseif msg == 'resize' then
   if debugMode then
      if ResizeBG() == true then
	  DEFAULT_CHAT_FRAME:AddMessage("|cFFFFAA00resized|r")
	  end
	end]]--
	elseif msg == 'print' then
      for  id, frame in ipairs(DefaultFrames) do -- All frames that are currently shown 
         local TempFrame = frame
         local TempFrameName = TempFrame:GetName()
        -- print(TempFrameName)
	  end   
	  
	  --elseif msg == 'h' then
		--for k,v in ipairs(BGS_FrameInfoTable.ExtraInfoFunctions.functions) do
		--	v.func;
		--end
	else
		if ( not InterfaceOptionsFramePanelContainer.displayedPanel ) then
			InterfaceOptionsFrame_OpenToCategory(CONTROLS_LABEL);
		end
		InterfaceOptionsFrame_OpenToCategory(addonName) 
		
	  
   end
end
SlashCmdList["PVPSTATS"] = slashcmd