local AceGUI = LibStub("AceGUI-3.0")
local addonName, BGS_FrameInfoTable = ...
local _frameName = "SessionFrame"
local _detail = "Current Session"
local _icon = [[Interface\ICONS\INV_Misc_PocketWatch_01]]
local _info = "Battleground wins and losses on this character this session (auto resets after buffer)"
local _options = {}
local _logoutTime = 600;
local _OptionList = {}

local scrollcontainer = AceGUI:Create("SimpleGroup") -- "InlineGroup" is also good
scrollcontainer:SetFullWidth(true)
scrollcontainer:SetWidth(185)
scrollcontainer:SetFullHeight(true) -- probably?
scrollcontainer:SetHeight(170)
scrollcontainer:SetLayout("Fill") -- important!

table.insert(_options, scrollcontainer)

	local scroll = AceGUI:Create("ScrollFrame")
	scroll:SetLayout("Flow") -- probably?
	scrollcontainer:AddChild(scroll)

	local frameDetail = AceGUI:Create("SimpleGroup")
	frameDetail:SetFullWidth(true)
	frameDetail:SetHeight(35)
	frameDetail:SetLayout("Fill")
	frameDetail.bg = frameDetail.frame:CreateTexture("frameDetail_Tex")
	frameDetail.bg:SetTexture("Interface\\LFGFRAME\\UI-LFG-SEPARATOR")
	frameDetail.bg:SetHeight(frameDetail.frame:GetHeight()/2)
	frameDetail.bg:SetTexCoord(0, 168/256, 0, 34/128)
	frameDetail.bg:SetPoint("bottom", 0, -2)
	frameDetail.text = frameDetail.frame:CreateFontString(nil, nil, "GameFontHighlightSmall")
	frameDetail.text:SetHeight(frameDetail.frame:GetHeight())
	frameDetail.text:SetText(_info)
	frameDetail.text:SetPoint("topleft", 0, 0)
	frameDetail.text:SetPoint("topright", 0, 0)
	scroll:AddChild(frameDetail)

	local txt_BufferTime = AceGUI:Create("EditBox")
	txt_BufferTime.frame:Hide()
	txt_BufferTime:SetLabel("Buffer time after logout (sec)")
	txt_BufferTime:SetText(_logoutTime)
	txt_BufferTime:SetWidth(180)
	txt_BufferTime:SetPoint("topleft", optionFrame, "topleft", 0, 0)
	txt_BufferTime:SetCallback("OnEnterPressed", function()
		local input = tonumber(txt_BufferTime:GetText())
		if input == nil then
			txt_BufferTime:SetText(_logoutTime)
		else
			_logoutTime = input
		end
	end)
	_OptionList.txt_BufferTime = txt_BufferTime
	scroll:AddChild(txt_BufferTime)

	local btn_Reset = AceGUI:Create("Button")
	btn_Reset.frame:Hide()
	btn_Reset:SetText("Reset session")
	btn_Reset:SetWidth(180)
	btn_Reset:SetPoint("topleft", optionFrame, "topleft", 0, -txt_BufferTime.frame:GetHeight()-10)
	btn_Reset:SetCallback("OnClick", function() 
		local frame = BGS_FrameInfoTable:GetFrameByName(_frameName);
		frame.text:SetText("0 : 0") 
	end)
	scroll:AddChild(btn_Reset)


table.insert(BGS_FrameInfoTable, { name = _frameName		--10
	, detail = _detail
	, icon = _icon
	, info = _info
	, options = _options
	}
	)
	
local function UpdateSessionInfo(win)
	--print("updatesession")
	local frame = BGS_FrameInfoTable:GetFrameByName(_frameName);
	if win == nil then
		frame.text:SetText("0 : 0")
		return
	end
	
	local wins, losses = string.match(frame.text:GetText(), "(%d+) : (%d+)")
	if win then
		wins = wins + 1;
	elseif not win then
		losses = losses + 1;
	end
	frame.text:SetText(wins .. " : " .. losses)

end
	
local function getBgFaction()
	for i=1,40 do 
		local name = UnitBuff("player",i);
		if name == "Horde" then
			return 0
		elseif name == "Alliance" then
			return 1
		end
	end
	
	local pFaction = UnitFactionGroup("player")
	if pFaction == "Horde" then
		return 0
	elseif pFaction == "Alliance" then
		return 1
	end
	return nil
end
	
local function _SessionInfos()
	local _, itype = IsInInstance()
	if itype ~= "pvp" or GetBattlefieldInstanceExpiration() == 0 then
		return
	end
	local wFaction = GetBattlefieldWinner()
	if wFaction == nil then
		return
	end
	local playerWon = false
	if wFaction == getBgFaction() then
		playerWon = true
	end
	
	UpdateSessionInfo(playerWon)
end
	
local function eventHandle(self, event, addon)
	if not (BGS_FrameInfoTable:GetFrameByName(_frameName)) then
		return
	end
	
	if event == "UPDATE_BATTLEFIELD_STATUS"then
		_SessionInfos()
	return
	end
	if event == "ADDON_LOADED" then
		if addon ~= addonName then return end
		self:UnregisterEvent("ADDON_LOADED")
		
		
		local frame = BGS_FrameInfoTable:GetFrameByName(_frameName);
		frame.text:SetText("0 : 0")
		
		local infoList = BGS_FrameInfoTable:GetInfoList(_frameName)
		
		if not (infoList)then
			return
		end
		
		if infoList.logTime ~= nill then
			_logoutTime = infoList.logTime
			_OptionList.txt_BufferTime:SetText(_logoutTime)
		end
		
		if (time() - infoList.timestamp) < _logoutTime then
			frame.text:SetText(infoList.text)
		end
		return
	end
	
	if event == "PLAYER_LOGOUT" then
		local frame = BGS_FrameInfoTable:GetFrameByName(_frameName)
		table.insert(BGstats_ExtraFrameDataList, {frame = _frameName, timestamp = time(), logTime = _logoutTime, text = frame.text:GetText()})
	end
end

local _eventsFrame = CreateFrame("FRAME", "BGS_".._frameName.."Events");
_eventsFrame:RegisterEvent("PLAYER_LOGIN");
_eventsFrame:RegisterEvent("UPDATE_BATTLEFIELD_STATUS");
_eventsFrame:RegisterEvent("ADDON_LOADED");
_eventsFrame:RegisterEvent("PLAYER_LOGOUT");
_eventsFrame:SetScript("OnEvent", function(self, event, ...) eventHandle(self, event, ...) end)